@@ -196,16 +196,18 @@ typedef struct FONT_
   /* configuration options */
   TA_Progress_Func progress;
   void* progress_data;
+  TA_Info_Func info;
+  void* info_data;
   FT_UInt hinting_range_min;
   FT_UInt hinting_range_max;
   FT_UInt hinting_limit;
+  FT_UInt increase_x_height;
   FT_Bool pre_hinting;
-  FT_Bool increase_x_height;
   FT_Bool no_x_height_snapping;
   FT_Byte* x_height_snapping_exceptions;
-  FT_Bool ignore_permissions;
+  FT_Bool ignore_restrictions;
   FT_UInt fallback_script;
-  FT_Bool no_info;
+  FT_Bool symbol;
 } FONT;
 
 
