@@ -14,6 +14,7 @@
 
 
 #include "ta.h"
+#include <string.h>
 
 
 #undef MISSING
@@ -52,8 +53,15 @@ typedef struct Recorder_
   GLYPH* glyph; /* the current glyph */
   Hints_Record hints_record;
 
-  /* see explanations in `TA_sfnt_build_glyph_segments' */
+  /* some segments can `wrap around' */
+  /* a contour's start point like 24-25-26-0-1-2 */
+  /* (there can be at most one such segment per contour); */
+  /* later on we append additional records */
+  /* to split them into 24-26 and 0-2 */
   FT_UInt* wrap_around_segments;
+  FT_UInt num_wrap_around_segments;
+
+  FT_UShort num_stack_elements; /* the necessary stack depth so far */
 
   /* data necessary for strong point interpolation */
   FT_UInt* ip_before_points;
@@ -100,7 +108,8 @@ TA_adjust_point_index(Recorder* recorder,
 static FT_Byte*
 TA_sfnt_build_glyph_segments(SFNT* sfnt,
                              Recorder* recorder,
-                             FT_Byte* bufp)
+                             FT_Byte* bufp,
+                             FT_Bool optimize)
 {
   FONT* font = recorder->font;
   TA_GlyphHints hints = &font->loader->hints;
@@ -118,13 +127,12 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   FT_UInt nargs;
   FT_UInt num_segments;
 
-  FT_UInt* wrap_around_segment;
-  FT_UInt num_wrap_around_segments;
-
   FT_Bool need_words = 0;
 
   FT_Int n;
   FT_UInt i, j;
+  FT_UInt base;
+  FT_UInt num_packed_segments;
   FT_UInt num_storage;
   FT_UInt num_stack_elements;
   FT_UInt num_twilight_points;
@@ -133,24 +141,39 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   seg_limit = segments + axis->num_segments;
   num_segments = axis->num_segments;
 
-  /* some segments can `wrap around' */
-  /* a contour's start point like 24-25-26-0-1-2 */
-  /* (there can be at most one such segment per contour); */
-  /* we thus append additional records to split them into 24-26 and 0-2 */
-  wrap_around_segment = recorder->wrap_around_segments;
+  /* to pack the data in the bytecode more tightly, */
+  /* we store up to the first nine segments in nibbles if possible, */
+  /* using delta values */
+  base = 0;
+  num_packed_segments = 0;
   for (seg = segments; seg < seg_limit; seg++)
-    if (seg->first > seg->last)
-    {
-      /* the stored data is used later for edge linking */
-      *(wrap_around_segment++) = seg - segments;
-    }
+  {
+    FT_UInt first = seg->first - points;
+    FT_UInt last = seg->last - points;
 
-  num_wrap_around_segments = wrap_around_segment
-                             - recorder->wrap_around_segments;
-  num_segments += num_wrap_around_segments;
 
-  /* wrap-around segments are pushed with four arguments */
-  num_args = 2 * num_segments + 2 * num_wrap_around_segments + 2;
+    first = TA_adjust_point_index(recorder, first);
+    last = TA_adjust_point_index(recorder, last);
+
+    if (first - base >= 16)
+      break;
+    if (first > last || last - first >= 16)
+      break;
+    if (num_packed_segments == 9)
+      break;
+    num_packed_segments++;
+    base = last;
+  }
+
+  /* also handle wrap-around segments */
+  num_segments += recorder->num_wrap_around_segments;
+
+  /* wrap-around segments are pushed with four arguments; */
+  /* a segment stored in nibbles needs only one byte instead of two */
+  num_args = num_packed_segments
+             + 2 * (num_segments - num_packed_segments)
+             + 2 * recorder->num_wrap_around_segments
+             + 2;
 
   /* collect all arguments temporarily in an array (in reverse order) */
   /* so that we can easily split into chunks of 255 args */
@@ -164,13 +187,34 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   if (num_segments > 0xFF)
     need_words = 1;
 
+  /* the number of packed segments is indicated by the function number */
   if (recorder->glyph->num_components)
-    *(arg--) = bci_create_segments_composite;
+    *(arg--) = bci_create_segments_composite_0 + num_packed_segments;
   else
-    *(arg--) = bci_create_segments;
+    *(arg--) = bci_create_segments_0 + num_packed_segments;
   *(arg--) = num_segments;
 
-  for (seg = segments; seg < seg_limit; seg++)
+  base = 0;
+  for (seg = segments; seg < segments + num_packed_segments; seg++)
+  {
+    FT_UInt first = seg->first - points;
+    FT_UInt last = seg->last - points;
+    FT_UInt low_nibble;
+    FT_UInt high_nibble;
+
+
+    first = TA_adjust_point_index(recorder, first);
+    last = TA_adjust_point_index(recorder, last);
+
+    low_nibble = first - base;
+    high_nibble = last - first;
+
+    *(arg--) = 16 * high_nibble + low_nibble;
+
+    base = last;
+  }
+
+  for (seg = segments + num_packed_segments; seg < seg_limit; seg++)
   {
     FT_UInt first = seg->first - points;
     FT_UInt last = seg->last - points;
@@ -246,8 +290,14 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
     {
       nargs = (num_args - i > 255) ? 255 : num_args - i;
 
-      BCI(NPUSHW);
-      BCI(nargs);
+
+      if (optimize && nargs <= 8)
+        BCI(PUSHW_1 - 1 + nargs);
+      else
+      {
+        BCI(NPUSHW);
+        BCI(nargs);
+      }
       for (j = 0; j < nargs; j++)
       {
         BCI(HIGH(*arg));
@@ -262,8 +312,13 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
     {
       nargs = (num_args - i > 255) ? 255 : num_args - i;
 
-      BCI(NPUSHB);
-      BCI(nargs);
+      if (optimize && nargs <= 8)
+        BCI(PUSHB_1 - 1 + nargs);
+      else
+      {
+        BCI(NPUSHB);
+        BCI(nargs);
+      }
       for (j = 0; j < nargs; j++)
       {
         BCI(*arg);
@@ -282,7 +337,10 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   if (num_twilight_points > sfnt->max_twilight_points)
     sfnt->max_twilight_points = num_twilight_points;
 
-  num_stack_elements = ADDITIONAL_STACK_ELEMENTS + num_args;
+  /* both this function and `TA_emit_hints_record' */
+  /* push data onto the stack */
+  num_stack_elements = ADDITIONAL_STACK_ELEMENTS
+                       + recorder->num_stack_elements + num_args;
   if (num_stack_elements > sfnt->max_stack_elements)
     sfnt->max_stack_elements = num_stack_elements;
 
@@ -352,8 +410,16 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
         max = q;
     }
 
-    *(arg--) = TA_adjust_point_index(recorder, min);
-    *(arg--) = TA_adjust_point_index(recorder, max);
+    if (min > max)
+    {
+      *(arg--) = TA_adjust_point_index(recorder, max);
+      *(arg--) = TA_adjust_point_index(recorder, min);
+    }
+    else
+    {
+      *(arg--) = TA_adjust_point_index(recorder, min);
+      *(arg--) = TA_adjust_point_index(recorder, max);
+    }
 
     start = end + 1;
   }
@@ -373,8 +439,13 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
     {
       nargs = (num_args - i > 255) ? 255 : num_args - i;
 
-      BCI(NPUSHW);
-      BCI(nargs);
+      if (nargs <= 8)
+        BCI(PUSHW_1 - 1 + nargs);
+      else
+      {
+        BCI(NPUSHW);
+        BCI(nargs);
+      }
       for (j = 0; j < nargs; j++)
       {
         BCI(HIGH(*arg));
@@ -389,8 +460,13 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
     {
       nargs = (num_args - i > 255) ? 255 : num_args - i;
 
-      BCI(NPUSHB);
-      BCI(nargs);
+      if (nargs <= 8)
+        BCI(PUSHB_1 - 1 + nargs);
+      else
+      {
+        BCI(NPUSHB);
+        BCI(nargs);
+      }
       for (j = 0; j < nargs; j++)
       {
         BCI(*arg);
@@ -820,9 +896,10 @@ TA_add_hints_record(Hints_Record** hints_records,
 
 
 static FT_Byte*
-TA_sfnt_emit_hints_record(SFNT* sfnt,
-                          Hints_Record* hints_record,
-                          FT_Byte* bufp)
+TA_emit_hints_record(Recorder* recorder,
+                     Hints_Record* hints_record,
+                     FT_Byte* bufp,
+                     FT_Bool optimize)
 {
   FT_Byte* p;
   FT_Byte* endp;
@@ -831,7 +908,6 @@ TA_sfnt_emit_hints_record(SFNT* sfnt,
   FT_UInt i, j;
   FT_UInt num_arguments;
   FT_UInt num_args;
-  FT_UInt num_stack_elements;
 
 
   /* check whether any argument is larger than 0xFF */
@@ -853,8 +929,13 @@ TA_sfnt_emit_hints_record(SFNT* sfnt,
     {
       num_args = (num_arguments - i > 255) ? 255 : (num_arguments - i);
 
-      BCI(NPUSHW);
-      BCI(num_args);
+      if (optimize && num_args <= 8)
+        BCI(PUSHW_1 - 1 + num_args);
+      else
+      {
+        BCI(NPUSHW);
+        BCI(num_args);
+      }
       for (j = 0; j < num_args; j++)
       {
         BCI(*p);
@@ -872,8 +953,13 @@ TA_sfnt_emit_hints_record(SFNT* sfnt,
     {
       num_args = (num_arguments - i > 255) ? 255 : (num_arguments - i);
 
-      BCI(NPUSHB);
-      BCI(num_args);
+      if (optimize && num_args <= 8)
+        BCI(PUSHB_1 - 1 + num_args);
+      else
+      {
+        BCI(NPUSHB);
+        BCI(num_args);
+      }
       for (j = 0; j < num_args; j++)
       {
         BCI(*p);
@@ -882,19 +968,20 @@ TA_sfnt_emit_hints_record(SFNT* sfnt,
     }
   }
 
-  num_stack_elements = ADDITIONAL_STACK_ELEMENTS + num_arguments;
-  if (num_stack_elements > sfnt->max_stack_elements)
-    sfnt->max_stack_elements = num_stack_elements;
+  /* collect stack depth data */
+  if (num_arguments > recorder->num_stack_elements)
+    recorder->num_stack_elements = num_arguments;
 
   return bufp;
 }
 
 
 static FT_Byte*
-TA_sfnt_emit_hints_records(SFNT* sfnt,
-                           Hints_Record* hints_records,
-                           FT_UInt num_hints_records,
-                           FT_Byte* bufp)
+TA_emit_hints_records(Recorder* recorder,
+                      Hints_Record* hints_records,
+                      FT_UInt num_hints_records,
+                      FT_Byte* bufp,
+                      FT_Bool optimize)
 {
   FT_UInt i;
   Hints_Record* hints_record;
@@ -918,21 +1005,17 @@ TA_sfnt_emit_hints_records(SFNT* sfnt,
     }
     BCI(LT);
     BCI(IF);
-    bufp = TA_sfnt_emit_hints_record(sfnt, hints_record, bufp);
+    bufp = TA_emit_hints_record(recorder, hints_record, bufp, optimize);
     BCI(ELSE);
 
     hints_record++;
   }
 
-  bufp = TA_sfnt_emit_hints_record(sfnt, hints_record, bufp);
+  bufp = TA_emit_hints_record(recorder, hints_record, bufp, optimize);
 
   for (i = 0; i < num_hints_records - 1; i++)
     BCI(EIF);
 
-  BCI(PUSHB_1);
-  BCI(bci_hint_glyph);
-  BCI(CALL);
-
   return bufp;
 }
 
@@ -1053,8 +1136,6 @@ TA_hints_recorder(TA_Action action,
   FT_UInt* wraps = recorder->wrap_around_segments;
   FT_Byte* p = recorder->hints_record.buf;
 
-  FT_Byte bound_offset = 0;
-
   FT_UInt* ip;
   FT_UInt* limit;
 
@@ -1150,21 +1231,16 @@ TA_hints_recorder(TA_Action action,
 
   case ta_bound:
     /* we ignore the BOUND action since we signal this information */
-    /* with the `bound_offset' parameter below */
+    /* with the proper function number */
     return;
 
   default:
     break;
   }
 
-  if (lower_bound)
-    bound_offset += 1;
-  if (upper_bound)
-    bound_offset += 2;
-
-  /* this reflects the order in the TA_Action enumeration */
-  *(p++) = 0;
-  *(p++) = (FT_Byte)action + bound_offset + ACTION_OFFSET;
+  /* some enum values correspond to four or eight bytecode functions; */
+  /* if the value is n, the function numbers are n, ..., n+7, */
+  /* to be differentiated with flags */
 
   switch (action)
   {
@@ -1175,9 +1251,10 @@ TA_hints_recorder(TA_Action action,
 
 
       *(p++) = 0;
-      *(p++) = stem_edge->flags & TA_EDGE_SERIF;
-      *(p++) = 0;
-      *(p++) = base_edge->flags & TA_EDGE_ROUND;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET
+               + ((stem_edge->flags & TA_EDGE_SERIF) != 0)
+               + 2 * ((base_edge->flags & TA_EDGE_ROUND) != 0);
+
       *(p++) = HIGH(base_edge->first - segments);
       *(p++) = LOW(base_edge->first - segments);
       *(p++) = HIGH(stem_edge->first - segments);
@@ -1194,9 +1271,10 @@ TA_hints_recorder(TA_Action action,
 
 
       *(p++) = 0;
-      *(p++) = edge2->flags & TA_EDGE_SERIF;
-      *(p++) = 0;
-      *(p++) = edge->flags & TA_EDGE_ROUND;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET
+               + ((edge2->flags & TA_EDGE_SERIF) != 0)
+               + 2 * ((edge->flags & TA_EDGE_ROUND) != 0);
+
       *(p++) = HIGH(edge->first - segments);
       *(p++) = LOW(edge->first - segments);
       *(p++) = HIGH(edge2->first - segments);
@@ -1214,9 +1292,11 @@ TA_hints_recorder(TA_Action action,
 
 
       *(p++) = 0;
-      *(p++) = edge2->flags & TA_EDGE_SERIF;
-      *(p++) = 0;
-      *(p++) = edge->flags & TA_EDGE_ROUND;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET
+               + ((edge2->flags & TA_EDGE_SERIF) != 0)
+               + 2 * ((edge->flags & TA_EDGE_ROUND) != 0)
+               + 4 * (edge_minus_one != NULL);
+
       *(p++) = HIGH(edge->first - segments);
       *(p++) = LOW(edge->first - segments);
       *(p++) = HIGH(edge2->first - segments);
@@ -1238,6 +1318,9 @@ TA_hints_recorder(TA_Action action,
       TA_Edge blue = arg2;
 
 
+      *(p++) = 0;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET;
+
       *(p++) = HIGH(blue->first - segments);
       *(p++) = LOW(blue->first - segments);
 
@@ -1267,9 +1350,11 @@ TA_hints_recorder(TA_Action action,
 
 
       *(p++) = 0;
-      *(p++) = edge2->flags & TA_EDGE_SERIF;
-      *(p++) = 0;
-      *(p++) = edge->flags & TA_EDGE_ROUND;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET
+               + ((edge2->flags & TA_EDGE_SERIF) != 0)
+               + 2 * ((edge->flags & TA_EDGE_ROUND) != 0)
+               + 4 * (edge_minus_one != NULL);
+
       *(p++) = HIGH(edge->first - segments);
       *(p++) = LOW(edge->first - segments);
       *(p++) = HIGH(edge2->first - segments);
@@ -1291,6 +1376,9 @@ TA_hints_recorder(TA_Action action,
       TA_Edge edge = (TA_Edge)arg1;
 
 
+      *(p++) = 0;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET;
+
       if (edge->best_blue_is_shoot)
       {
         *(p++) = HIGH(CVT_BLUE_SHOOTS_OFFSET(font) + edge->best_blue_idx);
@@ -1315,6 +1403,11 @@ TA_hints_recorder(TA_Action action,
       TA_Edge base = serif->serif;
 
 
+      *(p++) = 0;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET
+               + (lower_bound != NULL)
+               + 2 * (upper_bound != NULL);
+
       *(p++) = HIGH(serif->first - segments);
       *(p++) = LOW(serif->first - segments);
       *(p++) = HIGH(base->first - segments);
@@ -1341,6 +1434,11 @@ TA_hints_recorder(TA_Action action,
       TA_Edge edge = (TA_Edge)arg1;
 
 
+      *(p++) = 0;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET
+               + (lower_bound != NULL)
+               + 2 * (upper_bound != NULL);
+
       *(p++) = HIGH(edge->first - segments);
       *(p++) = LOW(edge->first - segments);
 
@@ -1366,6 +1464,11 @@ TA_hints_recorder(TA_Action action,
       TA_Edge after = arg3;
 
 
+      *(p++) = 0;
+      *(p++) = (FT_Byte)action + ACTION_OFFSET
+               + (lower_bound != NULL)
+               + 2 * (upper_bound != NULL);
+
       *(p++) = HIGH(before->first - segments);
       *(p++) = LOW(before->first - segments);
       *(p++) = HIGH(edge->first - segments);
@@ -1390,7 +1493,7 @@ TA_hints_recorder(TA_Action action,
 
   default:
     /* there are more cases in the enumeration */
-    /* which are handled with the `bound_offset' parameter */
+    /* which are handled with flags */
     break;
   }
 
@@ -1401,7 +1504,6 @@ TA_hints_recorder(TA_Action action,
 
 static FT_Error
 TA_init_recorder(Recorder* recorder,
-                 FT_UInt wrap_around_size,
                  FONT* font,
                  GLYPH* glyph,
                  TA_GlyphHints hints)
@@ -1411,8 +1513,12 @@ TA_init_recorder(Recorder* recorder,
   TA_Point point_limit = points + hints->num_points;
   TA_Point point;
 
-  FT_UInt num_strong_points = 0;
+  TA_Segment segments = axis->segments;
+  TA_Segment seg_limit = segments + axis->num_segments;
+  TA_Segment seg;
 
+  FT_UInt num_strong_points = 0;
+  FT_UInt* wrap_around_segment;
 
   recorder->font = font;
   recorder->glyph = glyph;
@@ -1423,14 +1529,26 @@ TA_init_recorder(Recorder* recorder,
   recorder->ip_on_point_array = NULL;
   recorder->ip_between_point_array = NULL;
 
+  recorder->num_stack_elements = 0;
+
   /* no need to clean up allocated arrays in case of error; */
   /* this is handled later by `TA_free_recorder' */
 
+  recorder->num_wrap_around_segments = 0;
+  for (seg = segments; seg < seg_limit; seg++)
+    if (seg->first > seg->last)
+      recorder->num_wrap_around_segments++;
+
   recorder->wrap_around_segments =
-    (FT_UInt*)malloc(wrap_around_size * sizeof (FT_UInt));
+    (FT_UInt*)malloc(recorder->num_wrap_around_segments * sizeof (FT_UInt));
   if (!recorder->wrap_around_segments)
     return FT_Err_Out_Of_Memory;
 
+  wrap_around_segment = recorder->wrap_around_segments;
+  for (seg = segments; seg < seg_limit; seg++)
+    if (seg->first > seg->last)
+      *(wrap_around_segment++) = seg - segments;
+
   /* get number of strong points */
   for (point = points; point < point_limit; point++)
   {
@@ -1474,13 +1592,22 @@ TA_init_recorder(Recorder* recorder,
 }
 
 
+static void
+TA_reset_recorder(Recorder* recorder,
+                  FT_Byte* bufp)
+{
+  recorder->hints_record.buf = bufp;
+  recorder->hints_record.num_actions = 0;
+}
+
+
 static void
 TA_rewind_recorder(Recorder* recorder,
                    FT_Byte* bufp,
                    FT_UInt size)
 {
-  recorder->hints_record.buf = bufp + 2;
-  recorder->hints_record.num_actions = 0;
+  TA_reset_recorder(recorder, bufp);
+
   recorder->hints_record.size = size;
 
   /* We later check with MISSING (which expands to 0xFF bytes) */
@@ -1496,7 +1623,6 @@ TA_rewind_recorder(Recorder* recorder,
   memset(recorder->ip_between_point_array, 0xFF,
          recorder->num_segments * recorder->num_segments
          * recorder->num_strong_points * sizeof (FT_UInt));
-
 }
 
 
@@ -1532,20 +1658,28 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
 
   TA_GlyphHints hints;
 
-  FT_UInt num_hints_records;
-  Hints_Record* hints_records;
+  FT_UInt num_action_hints_records;
+  FT_UInt num_point_hints_records;
+  Hints_Record* action_hints_records;
+  Hints_Record* point_hints_records;
 
   Recorder recorder;
+  FT_UInt num_stack_elements;
+  FT_Bool optimize = 0;
 
   FT_Int32 load_flags;
   FT_UInt size;
 
+  FT_Byte* pos[3];
+
 
   /* XXX: right now, we abuse this flag to control */
   /*      the global behaviour of the auto-hinter */
   load_flags = font->fallback_script << 30;
+  load_flags |= 1 << 29; /* vertical hinting only */
+  /* values 0 and 6-20 compressed to 4 bits */
   if (font->increase_x_height)
-    load_flags |= 1 << 29;
+    load_flags |= (font->increase_x_height - 5) << 25;
   if (!font->pre_hinting)
     load_flags |= FT_LOAD_NO_SCALE;
 
@@ -1582,9 +1716,6 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   /* so that we can easily find the array length at reallocation time */
   memset(ins_buf, INS_A0, ins_len);
 
-  num_hints_records = 0;
-  hints_records = NULL;
-
   /* handle composite glyph */
   if (font->loader->gloader->base.num_subglyphs)
   {
@@ -1615,19 +1746,11 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     goto Done1;
   }
 
-  error = TA_init_recorder(&recorder, face->glyph->outline.n_contours,
-                           font, glyph, hints);
+  error = TA_init_recorder(&recorder, font, glyph, hints);
   if (error)
     goto Err;
 
-  bufp = TA_sfnt_build_glyph_segments(sfnt, &recorder, ins_buf);
-  if (!bufp)
-  {
-    error = FT_Err_Out_Of_Memory;
-    goto Err;
-  }
-
-  /* now we loop over a large range of pixel sizes */
+  /* loop over a large range of pixel sizes */
   /* to find hints records which get pushed onto the bytecode stack */
 
 #ifdef DEBUGGING
@@ -1640,24 +1763,29 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
       putc('=', stderr);
     fprintf(stderr, "\n\n");
 
-    ta_glyph_hints_dump_edges(_ta_debug_hints);
-    ta_glyph_hints_dump_segments(_ta_debug_hints);
-    ta_glyph_hints_dump_points(_ta_debug_hints);
   }
 #endif
 
-  /* we temporarily use `ins_buf' to record the current glyph hints, */
-  /* leaving two bytes at the beginning so that the number of actions */
-  /* can be inserted later on */
+  /* we temporarily use `ins_buf' to record the current glyph hints */
   ta_loader_register_hints_recorder(font->loader,
                                     TA_hints_recorder,
                                     (void*)&recorder);
 
+  num_action_hints_records = 0;
+  num_point_hints_records = 0;
+  action_hints_records = NULL;
+  point_hints_records = NULL;
+
   for (size = font->hinting_range_min;
        size <= font->hinting_range_max;
        size++)
   {
-    TA_rewind_recorder(&recorder, bufp, size);
+#ifdef DEBUGGING
+    int have_dumps = 0;
+#endif
+
+
+    TA_rewind_recorder(&recorder, ins_buf, size);
 
     error = FT_Set_Pixel_Sizes(face, size, size);
     if (error)
@@ -1670,38 +1798,74 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     if (error)
       goto Err;
 
-    /* append the point hints data collected in `TA_hints_recorder' */
-    TA_build_point_hints(&recorder, hints);
-
-    /* store the number of actions in `ins_buf' */
-    *bufp = HIGH(recorder.hints_record.num_actions);
-    *(bufp + 1) = LOW(recorder.hints_record.num_actions);
-
-    if (TA_hints_record_is_different(hints_records,
-                                     num_hints_records,
-                                     bufp, recorder.hints_record.buf))
+    if (TA_hints_record_is_different(action_hints_records,
+                                     num_action_hints_records,
+                                     ins_buf, recorder.hints_record.buf))
     {
 #ifdef DEBUGGING
       {
-        fprintf(stderr, "  %d:\n", size);
-        for (p = bufp; p < recorder.hints_record.buf; p += 2)
+        have_dumps = 1;
+
+        fprintf(stderr, "  size %d:\n", size);
+
+        ta_glyph_hints_dump_edges(_ta_debug_hints);
+        ta_glyph_hints_dump_segments(_ta_debug_hints);
+        ta_glyph_hints_dump_points(_ta_debug_hints);
+
+        fprintf(stderr, "  action hints record:\n");
+        for (p = ins_buf; p < recorder.hints_record.buf; p += 2)
           fprintf(stderr, " %2d", *p * 256 + *(p + 1));
         fprintf(stderr, "\n");
       }
 #endif
 
-      error = TA_add_hints_record(&hints_records,
-                                  &num_hints_records,
-                                  bufp, recorder.hints_record);
+      error = TA_add_hints_record(&action_hints_records,
+                                  &num_action_hints_records,
+                                  ins_buf, recorder.hints_record);
+      if (error)
+        goto Err;
+    }
+
+    /* now handle point records */
+
+    TA_reset_recorder(&recorder, ins_buf);
+
+    /* use the point hints data collected in `TA_hints_recorder' */
+    TA_build_point_hints(&recorder, hints);
+
+    if (TA_hints_record_is_different(point_hints_records,
+                                     num_point_hints_records,
+                                     ins_buf, recorder.hints_record.buf))
+    {
+#ifdef DEBUGGING
+      {
+        if (!have_dumps)
+        {
+          fprintf(stderr, "  size %d:\n", size);
+
+          ta_glyph_hints_dump_edges(_ta_debug_hints);
+          ta_glyph_hints_dump_segments(_ta_debug_hints);
+          ta_glyph_hints_dump_points(_ta_debug_hints);
+        }
+
+        fprintf(stderr, "  point hints record:\n");
+        for (p = ins_buf; p < recorder.hints_record.buf; p += 2)
+          fprintf(stderr, " %2d", *p * 256 + *(p + 1));
+        fprintf(stderr, "\n");
+      }
+#endif
+
+      error = TA_add_hints_record(&point_hints_records,
+                                  &num_point_hints_records,
+                                  ins_buf, recorder.hints_record);
       if (error)
         goto Err;
     }
   }
 
-  if (num_hints_records == 1 && !hints_records[0].num_actions)
+  if (num_action_hints_records == 1 && !action_hints_records[0].num_actions)
   {
-    /* since we only have a single empty record we just scale the glyph, */
-    /* overwriting the data from `TA_sfnt_build_glyph_segments' */
+    /* since we only have a single empty record we just scale the glyph */
     bufp = TA_sfnt_build_glyph_scaler(sfnt, &recorder, ins_buf);
     if (!bufp)
     {
@@ -1717,20 +1881,149 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     goto Done;
   }
 
-  /* in most cases, the output of `TA_sfnt_build_glyph_segments' */
-  /* is shorter than the previously stored data, */
-  /* so clear the rest of the temporarily used part of `ins_buf' */
-  /* before appending the hints records */
+  /* if there is only a single record, */
+  /* we do a global optimization later on */
+  if (num_action_hints_records > 1)
+    optimize = 1;
+
+  /* store the hints records and handle stack depth */
+  pos[0] = ins_buf;
+  bufp = TA_emit_hints_records(&recorder,
+                               point_hints_records,
+                               num_point_hints_records,
+                               ins_buf,
+                               optimize);
+
+  num_stack_elements = recorder.num_stack_elements;
+  recorder.num_stack_elements = 0;
+
+  pos[1] = bufp;
+  bufp = TA_emit_hints_records(&recorder,
+                               action_hints_records,
+                               num_action_hints_records,
+                               bufp,
+                               optimize);
+
+  recorder.num_stack_elements += num_stack_elements;
+
+  pos[2] = bufp;
+  bufp = TA_sfnt_build_glyph_segments(sfnt, &recorder, bufp, optimize);
+  if (!bufp)
+  {
+    error = FT_Err_Out_Of_Memory;
+    goto Err;
+  }
+
+  /* XXX improve handling of NPUSHW */
+  if (num_action_hints_records == 1
+      && *(pos[0]) != NPUSHW && *(pos[1]) != NPUSHW && *(pos[2]) != NPUSHW)
+  {
+    /*
+     * we optimize two common cases, replacing
+     *
+     *   NPUSHB A ... NPUSHB B [... NPUSHB C] ... CALL
+     *
+     * with
+     *
+     *   NPUSHB (A+B[+C]) ... CALL
+     *
+     * if possible
+     */
+    FT_Byte sizes[3];
+    FT_Byte new_size1;
+    FT_Byte new_size2;
+
+    FT_UInt sum;
+    FT_UInt i;
+    FT_UInt pos_idx;
+
+
+    /* the point hints records block can be missing */
+    if (pos[0] == pos[1])
+    {
+      pos[1] = pos[2];
+      pos[2] = NULL;
+    }
+
+    /* there are at least two NPUSHB instructions */
+    /* (one of them directly at the start) */
+    sizes[0] = *(pos[0] + 1);
+    sizes[1] = *(pos[1] + 1);
+    sizes[2] = pos[2] ? *(pos[2] + 1) : 0;
+
+    sum = sizes[0] + sizes[1] + sizes[2];
+
+    if (sum > 2 * 0xFF)
+      goto Done2; /* nothing to do since we need three NPUSHB */
+    else if (!sizes[2] && (sum > 0xFF))
+      goto Done2; /* nothing to do since we need two NPUSHB */
+
+    if (sum > 0xFF)
+    {
+      /* reduce three NPUSHB to two */
+      new_size1 = 0xFF;
+      new_size2 = sum - 0xFF;
+    }
+    else
+    {
+      /* reduce two or three NPUSHB to one */
+      new_size1 = sum;
+      new_size2 = 0;
+    }
+
+    /* pack data */
+    p = ins_buf;
+    bufp = ins_buf;
+    pos_idx = 0;
+
+    if (new_size1 <= 8)
+      BCI(PUSHB_1 - 1 + new_size1);
+    else
+    {
+      BCI(NPUSHB);
+      BCI(new_size1);
+    }
+    for (i = 0; i < new_size1; i++)
+    {
+      if (p == pos[pos_idx])
+      {
+        pos_idx++;
+        p += 2; /* skip old NPUSHB */
+      }
+      *(bufp++) = *(p++);
+    }
+
+    if (new_size2)
+    {
+      if (new_size2 <= 8)
+        BCI(PUSHB_1 - 1 + new_size2);
+      else
+      {
+        BCI(NPUSHB);
+        BCI(new_size2);
+      }
+      for (i = 0; i < new_size2; i++)
+      {
+        if (p == pos[pos_idx])
+        {
+          pos_idx++;
+          p += 2;
+        }
+        *(bufp++) = *(p++);
+      }
+    }
+
+    BCI(CALL);
+  }
+
+Done2:
+  /* clear the rest of the temporarily used part of `ins_buf' */
   p = bufp;
   while (*p != INS_A0)
     *(p++) = INS_A0;
 
-  bufp = TA_sfnt_emit_hints_records(sfnt,
-                                    hints_records, num_hints_records,
-                                    bufp);
-
 Done:
-  TA_free_hints_records(hints_records, num_hints_records);
+  TA_free_hints_records(action_hints_records, num_action_hints_records);
   TA_free_recorder(&recorder);
 
   /* we are done, so reallocate the instruction array to its real size */
@@ -1760,7 +2053,7 @@ Done1:
   return FT_Err_Ok;
 
 Err:
-  TA_free_hints_records(hints_records, num_hints_records);
+  TA_free_hints_records(action_hints_records, num_action_hints_records);
   TA_free_recorder(&recorder);
   free(ins_buf);
 
