@@ -515,11 +515,13 @@ ta_loader_load_glyph(TA_Loader loader,
   scaler.flags = 0; /* XXX: fix this */
 
   /* XXX this is an ugly hack of ttfautohint: */
-  /* bit 30 and 31 of `load_flags' specify the fallback script, */
-  /* and bit 29 holds the `ignore_x_height' value */
+  /* bits 30 and 31 of `load_flags' specify the fallback script, */
+  /* bit 29 triggers vertical hinting only, */
+  /* and bits 25-28 give the x height increase limit */
   fallback_script = load_flags >> 30;
   if (load_flags & (1 << 29))
-    scaler.flags |= TA_SCALER_FLAG_INCREASE_X_HEIGHT;
+    scaler.flags |= TA_SCALER_FLAG_NO_HORIZONTAL;
+  scaler.flags |= ((load_flags >> 25) & 15) << 3; /* bits 3-6 */
 
   /* note that the fallback script can't be changed anymore */
   /* after the first call of `ta_loader_load_glyph' */
