@@ -26,7 +26,23 @@
 #include FT_OUTLINE_H
 
 
+/* enable one of the following three definitions for debugging */
 /* #define TA_DEBUG */
+/* #define TA_DEBUG_HORZ */
+/* #define TA_DEBUG_VERT */
+
+#if defined TA_DEBUG_HORZ
+#  define TA_DEBUG_STARTDIM TA_DIMENSION_HORZ
+#  define TA_DEBUG_ENDDIM TA_DIMENSION_HORZ
+#  define TA_DEBUG
+#elif defined TA_DEBUG_VERT
+#  define TA_DEBUG_STARTDIM TA_DIMENSION_VERT
+#  define TA_DEBUG_ENDDIM TA_DIMENSION_VERT
+#  define TA_DEBUG
+#elif defined TA_DEBUG
+#  define TA_DEBUG_STARTDIM TA_DIMENSION_VERT
+#  define TA_DEBUG_ENDDIM TA_DIMENSION_HORZ
+#endif
 
 #ifdef TA_DEBUG
 
@@ -110,10 +126,11 @@ typedef struct TA_GlyphHintsRec_* TA_GlyphHints;
 #define TA_SCALER_FLAG_NO_ADVANCE 0x04 /* disable advance hinting */
 
 /* for ttfautohint, we add more scaler flags; */
-/* note that FreeType uses a different mechanism */
+/* note that FreeType will use a different mechanism */
 /* to pass control arguments to the auto-hinter */
-/* (this isn't implemented yet in version 2.4.8) */
-#define TA_SCALER_FLAG_INCREASE_X_HEIGHT 0x08
+/* (this isn't implemented yet in version 2.4.9) */
+
+/* currently, bits 3-6 hold the x height increase limit */
 
 typedef struct TA_ScalerRec_
 {
