@@ -50,15 +50,17 @@ TTF_autohint(const char* options,
   FT_Long hinting_range_min = -1;
   FT_Long hinting_range_max = -1;
   FT_Long hinting_limit = -1;
+  FT_Long increase_x_height = -1;
 
   TA_Progress_Func progress;
   void* progress_data;
+  TA_Info_Func info;
+  void* info_data;
 
-  FT_Bool ignore_permissions = 0;
+  FT_Bool ignore_restrictions = 0;
   FT_Bool pre_hinting = 0;
-  FT_Bool no_info = 0;
-  FT_Bool increase_x_height = 0;
   FT_UInt fallback_script = 0;
+  FT_Bool symbol = 0;
 
   const char* op;
 
@@ -109,8 +111,8 @@ TTF_autohint(const char* options,
       hinting_range_max = (FT_Long)va_arg(ap, FT_UInt);
     else if (COMPARE("hinting-range-min"))
       hinting_range_min = (FT_Long)va_arg(ap, FT_UInt);
-    else if (COMPARE("ignore-permissions"))
-      ignore_permissions = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("ignore-restrictions"))
+      ignore_restrictions = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("in-buffer"))
     {
       in_file = NULL;
@@ -128,14 +130,16 @@ TTF_autohint(const char* options,
       in_len = 0;
     }
     else if (COMPARE("increase-x-height"))
-      increase_x_height = (FT_Bool)va_arg(ap, FT_Int);
+      increase_x_height = (FT_Long)va_arg(ap, FT_UInt);
     else if (COMPARE("out-buffer"))
     {
       out_file = NULL;
       out_bufp = va_arg(ap, char**);
     }
-    else if (COMPARE("no-info"))
-      no_info = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("info-callback"))
+      info = va_arg(ap, TA_Info_Func);
+    else if (COMPARE("info-callback-data"))
+      info_data = va_arg(ap, void*);
     else if (COMPARE("out-buffer-len"))
     {
       out_file = NULL;
@@ -153,6 +157,8 @@ TTF_autohint(const char* options,
       progress = va_arg(ap, TA_Progress_Func);
     else if (COMPARE("progress-callback-data"))
       progress_data = va_arg(ap, void*);
+    else if (COMPARE("symbol"))
+      symbol = (FT_Bool)va_arg(ap, FT_Int);
 
     /*
       x-height-snapping-exceptions
@@ -214,19 +220,31 @@ TTF_autohint(const char* options,
   if (hinting_limit < 0)
     hinting_limit = TA_HINTING_LIMIT;
 
+  /* we have 4 bits to store the values 0 and 6-20 */
+  if (increase_x_height > 0
+      && (increase_x_height < 6 || increase_x_height > 20))
+  {
+    error = FT_Err_Invalid_Argument;
+    goto Err1;
+  }
+  if (increase_x_height < 0)
+    increase_x_height = TA_INCREASE_X_HEIGHT;
+
   font->hinting_range_min = (FT_UInt)hinting_range_min;
   font->hinting_range_max = (FT_UInt)hinting_range_max;
   font->hinting_limit = (FT_UInt)hinting_limit;
+  font->increase_x_height = increase_x_height;
 
   font->progress = progress;
   font->progress_data = progress_data;
+  font->info = info;
+  font->info_data = info_data;
 
-  font->ignore_permissions = ignore_permissions;
+  font->ignore_restrictions = ignore_restrictions;
   font->pre_hinting = pre_hinting;
-  font->no_info = no_info;
-  font->increase_x_height = increase_x_height;
   /* restrict value to two bits */
   font->fallback_script = fallback_script & 3;
+  font->symbol = symbol;
 
   /* now start with processing the data */
 
@@ -241,7 +259,7 @@ TTF_autohint(const char* options,
     /* a valid TTF can never be that small */
     if (in_len < 100)
     {
-      error = FT_Err_Invalid_Argument;
+      error = TA_Err_Invalid_Font_Type;
       goto Err1;
     }
     font->in_buf = (FT_Byte*)in_buf;
@@ -292,7 +310,7 @@ TTF_autohint(const char* options,
 
       /* check lower byte of the `fsType' field */
       if (OS2_table->buf[OS2_FSTYPE_OFFSET + 1] == 0x02
-          && !font->ignore_permissions)
+          && !font->ignore_restrictions)
       {
         error = TA_Err_Missing_Legal_Permission;
         goto Err;
@@ -357,7 +375,7 @@ TTF_autohint(const char* options,
         goto Err;
     }
 
-    if (!font->no_info)
+    if (font->info)
     {
       /* add info about ttfautohint to the version string */
       error = TA_sfnt_update_name_table(sfnt, font);
